window.Gleam.initSearch([{"doc":"lustre_limiter","title":"lustre_limiter","content":"&lt;h1 align=&quot;center&quot;&gt;Lustre Limiter&lt;/h1&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n  Bringing debounce and throttle utilities to Lustre for the erlang target.\n\n[![Package Version](https://img.shields.io/hexpm/v/lustre_limiter)](https://hex.pm/packages/lustre_limiter)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/lustre_limiter/)\n\n&lt;/div&gt;\n\n&lt;br /&gt;\n\nLustre Limiter is available on [Hex](https://hexdocs.pm/lustre_limiter). Add it to your project by using the gleam CLI.\n\n```sh\ngleam add lustre_limiter\n```\n\n# Acknowledgments\n\nThis package is a port from [elm-limiter](https://github.com/hayleigh-dot-dev/elm-limiter) written by [Hayleigh](https://github.com/hayleigh-dot-dev), who was also a huge help in written this Gleam implementation.\n\n## How to use\n\n&gt; In the example, the package as been imported as an alias for simplicity&#39;s sake.\n&gt;\n&gt; ```\n&gt; import lustre_limiter as limiter\n&gt; ```\n\nIn order to use a debouncer or throttler in your Lustre application, you will need a few things.\n&lt;br /&gt;\nFirst, your model needs to store them as following.\n\n```gleam\ntype Model {\n  Model(\n    // ...\n    debouncer: limiter.Limiter(Msg),\n    throttler: limiter.Limiter(Msg),\n  )\n}\n```\n\nOnce done, you then have to declare the relevant Lustre `Msg` that will be sent to your update function when a message has been allowed through, and initialise your model using those `Msg`.\n\n```gleam\npub type Msg {\n  // Debounce\n  GetInput(String)\n  DebounceMsg(limiter.Msg(Msg))\n  SearchFor(String)\n\n  // Throttle\n  GetClick\n  ThrottleMsg(limiter.Msg(Msg))\n  AcknowledgeClick\n}\n\nfn init(_) -&gt; #(Model, effect.Effect(Msg)) {\n  #(\n    Model(\n      // ...\n      debouncer: limiter.debounce(DebounceMsg, 500),\n      throttler: limiter.throttle(ThrottleMsg, 500),\n    ),\n    effect.none(),\n  )\n}\n```\n\nNow comes the `update` function. `GetInput(String)` and `GetClick` will be the two `Msg` attached to our html elements, so their job is to tell the limiter that there is a new event to be process. To do this, we use `limiter.push`. The first element passed to `limiter.push` is the `Msg` we want to receive when the limiter has completed its task and we can now modify our application accordingly, and the second argument is our limiter.\n\n```gleam\npub fn update(model: Model, msg: Msg) -&gt; #(Model, effect.Effect(Msg)) {\n  case msg {\n    // Debounce\n    GetInput(value) -&gt; {\n      let #(debouncer, effect) =\n        limiter.push(SearchFor(value), model.debouncer)\n      #(\n        Model(\n          // ...\n        ),\n        effect,\n      )\n    }\n    // ...\n\n    // Throttle\n    GetClick -&gt; {\n      let #(throttler, effect) =\n        limiter.push(AcknowledgeClick, model.throttler)\n      #(\n        Model(\n          // ...\n        ),\n        effect,\n      )\n    }\n  }\n}\n```\n\nSince we wrapped our Lustre `Msg` in either `DebounceMsg` or `ThrottleMsg` when initialising our model, our `update` function will receive either of these messages when `limiter.push` is done processing them. This _does not_ mean that we can update our application yet, as we first need to check if the message is allowed. To do so, we use `limiter.update` as followed:\n\n```gleam\npub fn update(model: Model, msg: Msg) -&gt; #(Model, effect.Effect(Msg)) {\n  case msg {\n    // Debounce\n    DebounceMsg(internal_msg) -&gt; {\n      let #(debouncer, effect) = limiter.update(internal_msg, model.debouncer)\n      #(Model(..model, debouncer: debouncer), effect)\n    }\n    // ...\n\n    // Throttle\n    ThrottleMsg(internal_msg) -&gt; {\n      let #(throttler, effect) = limiter.update(internal_msg, model.throttler)\n      #(Model(..model, throttler: throttler), effect)\n    }\n    // ...\n  }\n}\n```\n\nIf the message received is authorised by `limiter.update`, our Lustre `update` function will finally receive the `Msg` type we gave to `limiter.push`, either `SearchFor(value)`, or `AcknowledgeClick` in our case. These two can be handle any way you like.\n&lt;br /&gt;\nTo - for example - debounce an input field using the example above, you only have to declare an input in your `view` function and set its event attribute to be `event.on_input(GetInput)`.\n\n## Example\n\nYou can run the provided example by typing the following and head to [localhost:3000](http://localhost:3000), or read its source code in [/example](/example).\n\n```console\ncd example &amp;&amp; gleam run\n```\n","url":"index.html"},{"doc":"lustre_limiter/internals","title":"Limiter","content":"pub type Limiter(msg) {\n  Limiter(\n    tagger: fn(Msg(msg)) -&gt; msg,\n    mode: Mode(msg),\n    state: State,\n  )\n}\n\nLimiter(\n  tagger: fn(Msg(msg)) -&gt; msg,\n  mode: Mode(msg),\n  state: State,\n)\n\n\nSynonyms:\nlustre_limiter/internals.Limiter\nlustre_limiter/internals Limiter","url":"lustre_limiter/internals.html#Limiter"},{"doc":"lustre_limiter/internals","title":"Mode","content":"pub type Mode(msg) {\n  Debounce(Int, List(msg))\n  Throttle(Int)\n}\n The mode of the limiter.\n The `Debounce` mode will keep track of how many messages have been received\n in a particular burst. Whenever a message is added to the list, a check is\n performed some time in the future. If the list hasn&#39;t changed by then, the\n newest message in the list is emitted and the rest are discarded.\n \n The `Throttle` mode only needs to keep track of the interval at which messages\n are throttled.\n \n Both modes expect the time to be given in _milliseconds_.\nDebounce(Int, List(msg))\n\n\nThrottle(Int)\n\n\nSynonyms:\nlustre_limiter/internals.Mode\nlustre_limiter/internals Mode","url":"lustre_limiter/internals.html#Mode"},{"doc":"lustre_limiter/internals","title":"Msg","content":"pub type Msg(msg) {\n  Emit(msg)\n  EmitIfSettled(Int)\n  None\n  Reopen\n  Push(msg)\n}\n\nEmit(msg)\n\n\nEmitIfSettled(Int)\n\n\nNone\n\n\nReopen\n\n\nPush(msg)\n\n\nSynonyms:\nlustre_limiter/internals.Msg\nlustre_limiter/internals Msg","url":"lustre_limiter/internals.html#Msg"},{"doc":"lustre_limiter/internals","title":"State","content":"pub type State {\n  Open\n  Close\n}\n\nOpen\n\n\nClose\n\n\nSynonyms:\nlustre_limiter/internals.State\nlustre_limiter/internals State","url":"lustre_limiter/internals.html#State"},{"doc":"lustre_limiter/internals","title":"emit","content":"pub fn emit(message: a) -&gt; Effect(a)\n Dispatch a message immediately.\nSynonyms:\nlustre_limiter/internals.emit\nlustre_limiter/internals emit","url":"lustre_limiter/internals.html#emit"},{"doc":"lustre_limiter/internals","title":"emit_after","content":"pub fn emit_after(message: a, delay: Int) -&gt; Effect(a)\n Take a message and dispatch it after a delay.\n This message will be handle by the `update` function\n of your Lustre application.\nSynonyms:\nlustre_limiter/internals.emit_after\nlustre_limiter/internals emit_after","url":"lustre_limiter/internals.html#emit_after"},{"doc":"lustre_limiter/internals","title":"lustre_limiter/internals","content":"","url":"lustre_limiter/internals.html"},{"doc":"lustre_limiter","title":"Limiter","content":"pub type Limiter(msg) =\n  internals.Limiter(msg)\n The limiter type.\n It holds data about the limiter, such as the tagger function, the mode and the state,\n so the limiter knows how to behave (.i.e. rate-limit incoming messages).\n One limiter is required per Lustre `Msg` you want to rate-limit, meaning it needs\n to be stored within the Lustre `Model`.\n\nSynonyms:\nlustre_limiter.Limiter\nlustre_limiter Limiter","url":"lustre_limiter.html#Limiter"},{"doc":"lustre_limiter","title":"Msg","content":"pub type Msg(msg) =\n  internals.Msg(msg)\n The message type for the limiter.\n This is what the limit uses internally to know what the `glimiter.update` function\n should do.\n\nSynonyms:\nlustre_limiter.Msg\nlustre_limiter Msg","url":"lustre_limiter.html#Msg"},{"doc":"lustre_limiter","title":"debounce","content":"pub fn debounce(\n  tagger: fn(Msg(a)) -&gt; a,\n  delay: Int,\n) -&gt; Limiter(a)\n Create a new debounce limiter.\n \n The way a debounce limiter works is by waiting for a certain amount of time\n after the last input before processing all the messages that were received.\n \n You can imagine this as a waiter in a restaurant. He waits as long as someone is talking\n and only when everyone has stopped talking for a certain amount of time,\n he walks to the kitchen to place the order.\n \n See [&#39;Debounce vs Throttle: Definitive Visual Guide&#39;](https://kettanaito.com/blog/debounce-vs-throttle) by Artem Zakharchenko\n for a more detailed explanation.\n \n ## Example:\n \n pub opaque type Model {\n   Model(\n      value: String,\n      debouncer: glimiter.Limiter(Msg),\n   )\n }\n \n pub fn init(_) -&gt; #(Model, effect.Effect(Msg)) {\n   #(\n     Model(\n       value: &quot;&quot;,\n       debouncer: glimiter.debounce(DebounceMsg, 500),\n     ),\n     effect.none(),\n   )\n }\n \n pub type Msg {\n   ...\n   DebounceMsg(glimiter.Msg(Msg))\n }\n \n \nSynonyms:\nlustre_limiter.debounce\nlustre_limiter debounce","url":"lustre_limiter.html#debounce"},{"doc":"lustre_limiter","title":"push","content":"pub fn push(\n  msg: a,\n  limiter: Limiter(a),\n) -&gt; #(Limiter(a), Effect(a))\n Push a message into the limiter.\n \n While we want to limit any action linked to an event - for example, fetching resources\n on the backend when the user types in a search bar - we don&#39;t want to limit the events\n themselves, otherwise we risk losing the user&#39;s input.\n \n To circumvent this, and to not make requests on every keypress, this function is used\n to manually push a message into the limiter.\n \n ## Example:\n \n pub fn update(model: Model, msg: Msg) -&gt; #(Model, effect.Effect(Msg)) {\n   case msg {\n     GetInput(value) -&gt; {\n       let #(debouncer, effect) =\n         glimiter.push(SearchFor(value), model.debouncer)\n       #(Model(value, debouncer, model.debounced_value), effect)\n    }\n   SearchFor(value) -&gt; {\n     // Perform whatever action needs to be done once the message is allowed through.\n   }\n }\n \n \nSynonyms:\nlustre_limiter.push\nlustre_limiter push","url":"lustre_limiter.html#push"},{"doc":"lustre_limiter","title":"throttle","content":"pub fn throttle(\n  tagger: fn(Msg(a)) -&gt; a,\n  interval: Int,\n) -&gt; Limiter(a)\n Create a new throttle limiter.\n \n Compared to a debouncer, the throttle acts as a valve, letting only one message\n through every `interval` milliseconds. This means that if you have a burst of messages\n coming in, the throttle will only let the first one through and then ignore the rest.\n \n See [&#39;Debounce vs Throttle: Definitive Visual Guide&#39;](https://kettanaito.com/blog/debounce-vs-throttle) by Artem Zakharchenko\n for a more detailed explanation.\n \n ## Example:\n \n pub opaque type Model {\n   Model(\n     value: String,\n     throttler: glimiter.Limiter(Msg),\n   )\n }\n \n pub fn init(_) -&gt; #(Model, effect.Effect(Msg)) {\n   #(\n     Model(\n       value: &quot;&quot;,\n       throttler: glimiter.throttle(500),\n     ),\n     effect.none(),\n   )\n }\n \n pub type Msg {\n  ThrottleMsg(glimiter.Msg(Msg))\n }\n \n \nSynonyms:\nlustre_limiter.throttle\nlustre_limiter throttle","url":"lustre_limiter.html#throttle"},{"doc":"lustre_limiter","title":"update","content":"pub fn update(\n  internal_msg: Msg(a),\n  limiter: Limiter(a),\n) -&gt; #(Limiter(a), Effect(a))\n Update the limiter.\n \n The limiter works using its own `Msg` values to update its internal state.\n When you create a limiter, you need to provide a &quot;tagger&quot; function that wraps these\n internal messages into a type that the Lustre `update` function can understand.\n \n This function should be called in the Lustre `update` when you get\n a &quot;wrapper&quot; message from the limiter. (i.e. `DebounceMsg` or `ThrottleMsg`).\n \n ## Example:\n \n pub fn update(model: Model, msg: Msg) -&gt; #(Model, effect.Effect(Msg)) {\n   case msg {\n    ...\n     DebounceMsg(internal_msg) -&gt; {\n       let #(debouncer, effect) = glimiter.update(internal_msg, model.debouncer) \n       #(Model(..model, debouncer: debouncer), effect)\n     }\n   }\n }\n \n \nSynonyms:\nlustre_limiter.update\nlustre_limiter update","url":"lustre_limiter.html#update"},{"doc":"lustre_limiter","title":"lustre_limiter","content":"","url":"lustre_limiter.html"}]);